#!/usr/bin/env bash

# fail on errors
set -e

die() {
  echo >&2 "$@"
  exit 1
}

# ensure root
[[ $EUID -ne 0 ]] && die "root is required to install on a system"

# parse args
while [[ $# -gt 0 ]]; do
  case $1 in
    -c|--nixos-config)
      NAME="$2"
      shift # past argument
      shift # past value
      ;;
    -k|--key-dir)
      KEYDIR="$2"
      shift # past argument
      shift # past value
      ;;
  esac
done

# check args
[ ! -f "$KEYDIR/host.pub" ] && die "host pubkey missing!"
[ ! -f "$KEYDIR/host" ] && die "host privkey missing!"
[ ! -f "$KEYDIR/host_initrd.pub" ] && die "host pubkey (initrd) missing!"
[ ! -f "$KEYDIR/host_initrd" ] && die "host privkey (initrd) missing!"
[ ! -f "$KEYDIR/luks-pw" ] && die "luks pw missing!"

# temp work dir
temp=$(mktemp -d)
cleanup() {
  rm -rf "$temp"
}
trap cleanup EXIT

# prepare host keys
echo "Preparing host keys.."
dir="$temp/persist/etc/ssh"
install -d -m755 "$dir"
cp "$KEYDIR/host" "$dir/ssh_host_ed25519_key"
cp "$KEYDIR/host.pub" "$dir/ssh_host_ed25519_key.pub"
chmod 600 "$dir/ssh_host_ed25519_key"

# prepare host keys (initrd)
echo "Preparing host keys.. (initrd)"
dir="$temp/persist/etc/secrets/initrd"
install -d -m755 "$dir"
cp "$KEYDIR/host" "$dir/ssh_host_ed25519_key"
cp "$KEYDIR/host.pub" "$dir/ssh_host_ed25519_key.pub"
chmod 600 "$dir/ssh_host_ed25519_key"

# run disko
ROOT_MNT="/mnt"
cp "$KEYDIR/luks-pw" "/tmp/luks-pw"
disko \
  --mode disko \
  --flake .#$NAME \
  --root-mountpoint $ROOT_MNT

# copy extra files into mountpoint
cp -ar "$temp/." "$ROOT_MNT/"

# fix permissions of mountpoint
chmod 755 "$ROOT_MNT"

# run nixos-install
nixos-install \
  --no-channel-copy \
  --no-root-password \
  --root "$ROOT_MNT" \
  --flake .#$NAME

echo -e "Finished install.\n" \
        "Make sure to delete the SSH host keys from here if you are done with them."
